defmodule MessengerBot.Client do
  @moduledoc """
  MessengerBot Client
  """

  alias MessengerBot.Config

  @base_url "https://graph.facebook.com/v2.6/me"
  @headers [{"Content-Type", "application/json"}]

  @doc """
  Set endpoint url
  """
  @spec url(String.t) :: String.t
  @spec url(String.t, String.t | nil) :: String.t
  def url(path, page_token \\ nil) do
    "#{@base_url}#{path}?access_token=#{page_token || Config.page_token()}"
  end

  @doc """
  Get request to messenger api
  """
  @spec get(String.t) ::
    HTTPoison.Response.t | {:error, HTTPoison.Error.t}
  def get(url) do
    case HTTPoison.get(url, @headers) do
      {:ok, resp} -> resp
      {:error, err} -> {:error, err}
    end
  end

  @doc """
  Post to messenger api
  """
  @spec post(String.t, Map.t | list()) ::
    HTTPoison.Response.t | {:error, HTTPoison.Error.t}
  def post(url, payload) do
    case HTTPoison.post(url, to_json(payload), @headers) do
      {:ok, resp} -> resp
      {:error, err} -> {:error, err}
    end
  end

  @doc """
  Post async to messenger api
  """
  @spec post_async(String.t, Map.t | list()) :: :ok
  def post_async(url, payload) do
    spawn(fn -> post(url, payload) end)
    :ok
  end

  @doc """
  Delete request to messenger api
  """
  @spec post(String.t, Map.t | list()) ::
    HTTPoison.Response.t | {:error, HTTPoison.Error.t}
  def delete(url, payload) do
    case HTTPoison.request(:delete, url, to_json(payload), @headers) do
      {:ok, resp} -> resp
      {:error, err} -> {:error, err}
    end
  end

  @doc """
  Delete request to messenger api
  """
  @spec delete_async(String.t, Map.t | list()) :: :ok
  def delete_async(url, payload) do
    spawn(fn -> delete(url, payload) end)
    :ok
  end

  defp to_json(payload) do
    payload = Poison.encode!(payload)
    Regex.replace(~r/\"([^\"]+)\":null(,?)/, payload, "")
  end
end
