defmodule MessengerBot.Service.MessagengerProfile do
  @moduledoc """
  Facebook Messenger 'Message' API
  """

  require Logger
  alias MessengerBot.Client
  alias MessengerBot.Config
  alias MessengerBot.Payload.{MessengerProfileSetRequest,
    MessengerProfileSetResponse, MessengerProfileReadResponse}

  @path "/messenger_profile"

  @doc """
  Setup get started button
  """
  @spec setup_get_started_button(String.t, MessengerProfileSetRequest.t) ::
    MessengerProfileSetResponse.t | {:error, Map.t}
  def setup_get_started_button(page_id, %MessengerProfileSetRequest{} = request) do
    page_token = Config.page_token(page_id)
    case Client.post(Client.url(@path, page_token), request) do
      %HTTPoison.Response{} = response ->
        Poison.decode!(response.body, as: %MessengerProfileSetResponse{})
      {:error, err} ->
        Logger.log(:info, fn -> inspect(err) end)
        {:error, %{msg: "Error returned from Facebook!", details: err}}
    end
  end

  @doc """
  Read get started button
  """
  @spec read_get_started_button(String.t) ::
    MessengerProfileReadResponse.t | {:error, Map.t}
  def read_get_started_button(page_id) do
    page_token = Config.page_token(page_id)
    url = Client.url(@path, page_token) <> "&fields=get_started"
    case Client.get(url) do
      %HTTPoison.Response{} = response ->
        Poison.decode!(response.body, as: %MessengerProfileReadResponse{})
      {:error, err} ->
        Logger.log(:info, fn -> inspect(err) end)
        {:error, %{details: err}}
    end
  end

  @doc """
  Delete get started button
  """
  @spec delete_get_started_button(String.t) ::
    MessengerProfileSetResponse.t | {:error, Map.t}
  def delete_get_started_button(page_id) do
    page_token = Config.page_token(page_id)
    request = %{fields: ["get_started"]}
    case Client.delete(Client.url(@path, page_token), request) do
      %HTTPoison.Response{} = response ->
        Poison.decode!(response.body, as: %MessengerProfileSetResponse{})
      {:error, err} ->
        Logger.log(:info, fn -> inspect(err) end)
        {:error, %{details: err}}
    end
  end

  @doc """
  Setup persistent menu
  """
  @spec setup_persistent_menu(String.t, MessengerProfileSetRequest.t) ::
    MessengerProfileSetResponse.t | {:error, Map.t}
  def setup_persistent_menu(page_id, %MessengerProfileSetRequest{} = request) do
    page_token = Config.page_token(page_id)
    case Client.post(Client.url(@path, page_token), request) do
      %HTTPoison.Response{} = response ->
        Poison.decode!(response.body, as: %MessengerProfileSetResponse{})
      {:error, err} ->
        Logger.log(:info, fn -> inspect(err) end)
        {:error, %{details: err}}
    end
  end

  @doc """
  Read persistent menu
  """
  @spec read_persistent_menu(String.t) ::
    MessengerProfileReadResponse.t | {:error, Map.t}
  def read_persistent_menu(page_id) do
    page_token = Config.page_token(page_id)
    url = Client.url(@path, page_token) <> "&fields=persistent_menu"
    case Client.get(url) do
      %HTTPoison.Response{} = response ->
        Poison.decode!(response.body, as: %MessengerProfileReadResponse{})
      {:error, err} ->
        Logger.log(:info, fn -> inspect(err) end)
        {:error, %{details: err}}
    end
  end

  @doc """
  Delete persistent menu
  """
  @spec delete_persistent_menu(String.t) ::
    MessengerProfileSetResponse.t | {:error, Map.t}
  def delete_persistent_menu(page_id) do
    page_token = Config.page_token(page_id)
    request = %{fields: ["persistent_menu"]}
    case Client.delete(Client.url(@path, page_token), request) do
      %HTTPoison.Response{} = response ->
        Poison.decode!(response.body, as: %MessengerProfileSetResponse{})
      {:error, err} ->
        Logger.log(:info, fn -> inspect(err) end)
        {:error, %{details: err}}
    end
  end
end
