defmodule MessengerBot.Webhook.Callback do
  @moduledoc """
  MessengerBot webhook for all callbacks
  """

  alias MessengerBot.Model.{Entry, Sender, Recipient, AccountLinking,
    Delivery, Message, Optin, Postback, Read, Referral}
  alias MessengerBot.Payload.Process, as: ProcessPayload
  alias MessengerBot.Payload.Webhook, as: WebhookPayload
  alias MessengerBot.Webhook.Worker.AccountLinking, as: AccountLinkingWorker
  alias MessengerBot.Webhook.Worker.Delivery, as: DeliveryWorker
  alias MessengerBot.Webhook.Worker.Message, as: MessageWorker
  alias MessengerBot.Webhook.Worker.Optin, as: OptinWorker
  alias MessengerBot.Webhook.Worker.Postback, as: PostbackWorker
  alias MessengerBot.Webhook.Worker.Read, as: ReadWorker
  alias MessengerBot.Webhook.Worker.Referral, as: ReferralWorker

  @decoder %WebhookPayload{entry: [%Entry{}]}

  @spec process(String.t) :: {:ok, String.t}
  def process(payload) do
    webhook_payload = Poison.decode!(payload, as: @decoder, keys: :atoms)
    Enum.each(webhook_payload.entry, fn(entry) -> process_entry(entry) end)
    {:ok, "ok"}
  end

  defp process_entry(%Entry{} = entry) do
    Enum.each(entry.messaging, fn(messaging) ->
      process_messaging(entry.id, entry.time, messaging)
    end)
  end

  defp process_messaging(page_id, time, messaging) do
    {worker, data} = fetch_worker_and_data(messaging)
    payload = %ProcessPayload{page_id: page_id, data: data, time: time,
      sender: struct(Sender, messaging.sender),
      recipient: struct(Recipient, messaging.recipient)}
    worker.perform(payload)
  end

  defp fetch_worker_and_data(%{account_linking: _} = messaging),
    do: {AccountLinkingWorker, struct(AccountLinking, messaging.account_linking)}
  defp fetch_worker_and_data(%{delivery: _} = messaging),
    do: {DeliveryWorker, struct(Delivery, messaging.delivery)}
  defp fetch_worker_and_data(%{message: _} = messaging),
    do: {MessageWorker, struct(Message, messaging.message)}
  defp fetch_worker_and_data(%{optin: _} = messaging),
    do: {OptinWorker, struct(Optin, messaging.optin)}
  defp fetch_worker_and_data(%{postback: _} = messaging),
    do: {PostbackWorker, struct(Postback, messaging.postback)}
  defp fetch_worker_and_data(%{read: _} = messaging),
    do: {ReadWorker, struct(Read, messaging.read)}
  defp fetch_worker_and_data(%{referral: _} = messaging),
    do: {ReferralWorker, struct(Referral, messaging.referral)}
end
